import 'package:collection/collection.dart';
import 'package:flutter/widgets.dart';
import 'package:leancode_common_flutter/leancode_common_flutter.dart';
import 'package:leancode_localization/src/localization.dart';
import 'package:leancode_localization/src/storages/locale_storage.dart';
import 'package:leancode_localization/src/translators/translator.dart';
import 'package:provider/provider.dart';

class AddLocalization extends StatefulWidget {
  const AddLocalization({
    Key key,
    this.initialLocale,
    @required this.child,
    @required this.storage,
    @required this.onLoading,
    @required this.translators,
  })  : assert(child != null),
        assert(storage != null),
        assert(onLoading != null),
        assert(translators != null),
        super(key: key);

  final Widget child;
  final Widget onLoading;
  final Locale initialLocale;
  final LocaleStorage storage;
  final List<Translator> translators;

  @override
  AddLocalizationState createState() {
    return AddLocalizationState();
  }
}

class AddLocalizationState extends State<AddLocalization> {
  Localization localization;

  static const collectionEquality = DeepCollectionEquality();

  @override
  void initState() {
    super.initState();
    localization = Localization(
      widget.storage,
      initialLocale: widget.initialLocale,
      translators: widget.translators,
    );
  }

  @override
  void didUpdateWidget(AddLocalization oldWidget) {
    super.didUpdateWidget(oldWidget);
    final oldLocales = oldWidget.translators.map((t) => t.locale);
    final locales = widget.translators.map((t) => t.locale);

    if (oldWidget.initialLocale != widget.initialLocale ||
        oldWidget.storage != widget.storage ||
        !collectionEquality.equals(oldLocales, locales)) {
      localization?.dispose();
      localization = Localization(
        widget.storage,
        initialLocale: widget.initialLocale,
        translators: widget.translators,
      );
    }
  }

  @override
  void dispose() {
    localization?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return AsyncInitializer(
      model: localization,
      onLoading: widget.onLoading,
      child: Provider<Localization>.value(
        value: localization,
        child: widget.child,
      ),
    );
  }
}
