import 'package:devicelocale/devicelocale.dart';
import 'package:flutter/widgets.dart';
import 'package:leancode_common/leancode_common.dart';
import 'package:leancode_localization/src/localization_delegate.dart';
import 'package:leancode_localization/src/storages/locale_storage.dart';
import 'package:leancode_localization/src/translators/translator.dart';
import 'package:rxdart/rxdart.dart';

class Localization implements Disposable, AsyncInitializable {
  Localization(
    this._storage, {
    Locale initialLocale,
    Iterable<Translator> translators,
  })  : _localeContoller = BehaviorSubject.seeded(initialLocale),
        _delegate = LocalizationDelegate(translators);

  final LocaleStorage _storage;
  final LocalizationDelegate _delegate;
  final BehaviorSubject<Locale> _localeContoller;

  LocalizationDelegate get delegate => _delegate;
  Iterable<Locale> get supportedLocales => _delegate.supportedLocales;
  Stream<Locale> get localeStream => _localeContoller.stream;
  Locale get initialLocale => _localeContoller.value;

  Future<void> changeLanguage(Locale locale) async {
    if (locale != _localeContoller.value) {
      await _storage.setLocale(locale);
      _localeContoller.add(locale);
    }
  }

  @override
  void dispose() {
    _localeContoller.close();
  }

  @override
  Future<void> initializeAsync() async {
    final systemLocaleCulture = await Devicelocale.currentLocale;
    final systemLocaleLang = systemLocaleCulture.substring(0, 2);
    final systemLocale = Locale(systemLocaleLang);

    const fallbackLocale = Locale('en');
    final userDefinedLocale = await _storage.getLocale();

    final locale = userDefinedLocale ?? delegate.isSupported(systemLocale)
        ? systemLocale
        : fallbackLocale;

    if (locale != _localeContoller.value) {
      _localeContoller.add(locale);
    }
  }

  static Locale localeFromContext(BuildContext context) {
    return Localizations.localeOf(context);
  }

  static Translator translator(BuildContext context) {
    return Localizations.of<Translator>(context, Translator);
  }

  static Translate translate(BuildContext context) {
    return Localization.translator(context).translate;
  }
}
