import 'dart:convert';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:leancode_localization/src/translators/translator.dart';
import 'package:meta/meta.dart';

class JsonTranslator extends Translator {
  JsonTranslator({
    @required Locale locale,
    @required this.fileName,
    this.returnKeyIfNotTranslated = false,
  })  : assert(fileName != null),
        assert(locale != null),
        _sentences = <String, String>{},
        super(locale);

  final String fileName;
  final Map<String, String> _sentences;
  final bool returnKeyIfNotTranslated;

  @override
  Future<void> load() async {
    final String data = await rootBundle.loadString(fileName);

    _sentences.clear();
    final Map<String, dynamic> result = json.decode(data);
    result.forEach((String key, dynamic value) {
      if (returnKeyIfNotTranslated == true && value.toString().isEmpty)
        _sentences[key] = key;
      else {
        _sentences[key] = value.toString();
      }
    });
  }

  @override
  String translate(String key) => _sentences[key];
}
